package icshttp

import (
	"crypto/rand"
	"encoding/json"
	"fmt"
	"io/ioutil"
	"math/big"
	"net/http"
	"strings"
	"sync"
	"time"

	"gitlab.com/cinnamon/voiceagent/icsconf"
	"gitlab.com/cinnamon/voiceagent/icslog"
	"gitlab.com/cinnamon/voiceagent/icspacketer"
	"gitlab.com/cinnamon/voiceagent/icspacketparser"
)

const HANDLE_NUM = 1024

const (
	TTS_FUNC1 = iota
	TTS_FUNC2
	TTS_FUNC3
	TTS_FUNC4
	BOT_FUNC
)

type Handler struct {
	r *Router
}

type handleInfo struct {
	id      int
	method  string
	pattern string
	h       http.HandlerFunc
}

type Request struct {
	Token          string          `json:"token"`
	OprMngCode     string          `json:"oprMngCode"`
	Method         string          `json:"method"`
	TalkText       string          `json:"talkText"`
	CallId         string          `json:"callId"`
	TrunkGroup     string          `json:"trunkGroup"`
	TelNo          string          `json:"telNo"`
	Dnis           string          `json:"dnis"`
	MentType       string          `json:"mentType"`
	RecordFilePath string          `json:"recordFilePath"`
	ServerID       string          `json:"serverId"`
	ServerIP       string          `json:"serverIp"`
	Data           json.RawMessage `json:"data"`
}

type Transrate struct {
	TargetDnis string `json:"target_dnis"`
}

type Response struct {
	ResultCode       int     `json:"resultCode"`
	Token            string  `json:"token"`
	Action           string  `json:"action"`
	AnounceMents     string  `json:"announceMents"`
	AnnounceFilePath string  `json:"announceFilePath"`
	Data             ResData `json:"DATA"`
}

type ResData struct {
	BargeIn      string          `json:"bargeIn"`
	RecodingFile string          `json:"recodingFile"`
	SttMaxTime   int             `json:"sttMaxTime"`
	MaxDigit     int             `json:"maxDigit"`
	MinDigit     int             `json:"minDigit"`
	DigitTerm    int             `json:"digitTerm"`
	TelNo        string          `json:"telNo"`
	VoiceName    int             `json:"voiceName"`
	Speed        int             `json:"speed"`
	Volume       int             `json:"volume"`
	Pitch        int             `json:"pitch"`
	EndCharacter []string        `json:"endCharacter"`
	MaxWaitTime  int             `json:"maxNoInputTime"`
	UUI          json.RawMessage `json:"uui"`
}

type ResStatus struct {
	Count         int
	Status        string
	NotUnderstand int
	PreEventNum   int
	PreEventNum2  int
	LoopCount     int
}

type ScenarioSession struct {
	session map[string]ResStatus
	m       *sync.Mutex
}

// var scenarioSession map[string]ResStatus

var handles []*handleInfo
var session []*ScenarioSession

func init() {
	handles = make([]*handleInfo, HANDLE_NUM)
	// session := make(map[string]ResStatus) // session init
	scenarioSession := new(ScenarioSession)
	scenarioSession.session = make(map[string]ResStatus, 200)
	scenarioSession.m = &sync.Mutex{}
	// session = make(map[string]ResStatus) // session init

	//TTS
	handles[TTS_FUNC1] = &handleInfo{method: "GET", pattern: "/tts/1", h: TTSFunc1}
	handles[TTS_FUNC2] = &handleInfo{method: "GET", pattern: "/tts/2", h: TTSFunc2}
	handles[TTS_FUNC3] = &handleInfo{method: "POST", pattern: "/tts/3", h: TTSFunc3}
	handles[TTS_FUNC4] = &handleInfo{method: "POST", pattern: "/tts/4", h: TTSFunc4}

	// bot sinario
	handles[BOT_FUNC] = &handleInfo{method: "POST", pattern: "/platform/api/call/process", h: scenarioSession.BOTPFunc}
}

func BuildHandler(r *Router) {
	for iter, handle := range handles {
		if handle != nil {
			//fmt.Println(handle)
			handle.id = iter
			r.HandleFunc(handle.method, handle.pattern, handle.h)
		} else {
			break
		}
	}
}

/////////////////////////////////////////////
//TTS

func TTSFunc1(w http.ResponseWriter, r *http.Request) {
	h := icspacketparser.NewHTTP()
	err := icspacketer.PutPacket(r, &h)
	if err != nil {
		fmt.Println(err)
	}

	fmt.Fprintln(w, "TTS func 1", r)
}

func TTSFunc2(w http.ResponseWriter, r *http.Request) {
	fmt.Fprintln(w, "TTS func 2", r)
}

// post
func TTSFunc3(w http.ResponseWriter, r *http.Request) {
	fmt.Fprintln(w, "TTS func 3", r)
	body := make([]byte, r.ContentLength)
	r.Body.Read(body)
	fmt.Println("TTS func 3", r.Method, r.URL)
	for iter, v := range r.Header {
		fmt.Println(iter, v)
	}
	fmt.Println(string(body))
}

// post
func TTSFunc4(w http.ResponseWriter, r *http.Request) {
	fmt.Fprintln(w, "TTS func 4", r)
}

// ///////////////////////////////////////////
// BOT
func (s *ScenarioSession) BOTPFunc(w http.ResponseWriter, r *http.Request) {
	// get scenario config
	l := icslog.GetIcsLog()
	conf := icsconf.GetIcsConfig()
	scnarioConf := conf.ScenarioConfig
	// Request json parsing
	request := Request{}
	resBody, err := ioutil.ReadAll(r.Body)
	if err != nil {
		fmt.Println(err)
	}
	defer r.Body.Close()

	err = json.Unmarshal(resBody, &request)
	fmt.Printf("%+v\n", request)
	if err != nil {
		l.Printf(icslog.LOG_LEVEL_INFO, -1, "response unmarshal error!!!!! ")
	}

	agent := strings.SplitN(request.CallId, "@", 2)
	l.Printf(icslog.LOG_LEVEL_INFO, -1, ">>>>>> DATA INFO - method [%s], agent [%s], token [%s] talk[%s]", request.Method, agent, request.Token, request.TalkText)

	response := new(Response)
	dnis := request.Dnis

	if request.Method == "HANGUP" {
		response.ResultCode = 200
		response.Token = request.Token
		response.Action = ""
		response.AnounceMents = ""
		response.Data.BargeIn = ""
		response.Data.RecodingFile = ""
		response.Data.SttMaxTime = 10
	} else if request.Method != "INIT" && request.TalkText == "" {
		l.Printf(icslog.LOG_LEVEL_INFO, -1, "여기는 INIT 아닐 때")
		// if request.TalkText == "" {
		// 	request.TalkText = "잘못된 값 입력 "
		// }

		// fmt.Println(dnis)
		// if dnis == "AISB" || dnis == "07075999956" || dnis == "821462" || dnis == "1462" {
		// 	dnis = "9013"
		// }
		// fmt.Println(dnis)
		// if dnis != "07012345678" && dnis != "07077442711" {
		// 	dnis = "sim"
		// }

		// if dnis == "07012345678" || dnis == "07077442711" {
		// 	dnis = "sim3"
		// }

		if dnis == "9012" {
			l.Printf(icslog.LOG_LEVEL_INFO, -1, "9012..s.session[request.Token].Count: %d", s.session[request.Token].Count)
			response.ResultCode = 200
			response.Token = request.Token
			response.Action = scnarioConf[dnis][s.session[request.Token].Count+1].Action
			response.AnounceMents = scnarioConf[dnis][s.session[request.Token].Count+1].AnounceMents
			if s.session[request.Token].Count != 0 && dnis == "9012" {
				response.AnounceMents = strings.ReplaceAll(response.AnounceMents, "$", request.TalkText)
			}
			response.Data.Speed = scnarioConf[dnis][s.session[request.Token].Count+1].Speed
			response.Data.VoiceName = scnarioConf[dnis][s.session[request.Token].Count+1].VoiceName
			response.Data.Volume = scnarioConf[dnis][s.session[request.Token].Count+1].Volume
			response.Data.Pitch = scnarioConf[dnis][s.session[request.Token].Count+1].Pitch
			response.Data.MaxWaitTime = scnarioConf[dnis][s.session[request.Token].Count+1].MaxWaitTime
			response.Data.BargeIn = scnarioConf[dnis][s.session[request.Token].Count+1].BargeIn
			response.Data.SttMaxTime = scnarioConf[dnis][s.session[request.Token].Count+1].SttMaxTime
			response.Data.MaxDigit = scnarioConf[dnis][s.session[request.Token].Count+1].MaxDigit
			response.Data.EndCharacter = []string{scnarioConf[dnis][s.session[request.Token].Count+1].EndCharacter}
			response.Data.DigitTerm = scnarioConf[dnis][s.session[request.Token].Count+1].DigitTerm
			response.Data.RecodingFile = "Y"

			s.session[request.Token] = ResStatus{Count: s.session[request.Token].Count + 1, Status: scnarioConf[dnis][s.session[request.Token].Count+1].Action}
			l.Printf(icslog.LOG_LEVEL_INFO, -1, "response.scnarioConf[dnis][s.session[request.Token].Count+1].AnounceMents: %s", scnarioConf[dnis][s.session[request.Token].Count+1].AnounceMents)
			l.Printf(icslog.LOG_LEVEL_INFO, -1, "response.AnounceMents: %s", response.AnounceMents)

		} else if dnis == "sim3" {
			//시뮬레이터
			response.ResultCode = 200
			response.Action = scnarioConf[dnis][s.session[request.Token].Count].Action
			response.AnounceMents = scnarioConf[dnis][s.session[request.Token].Count].AnounceMents
			response.Data.Speed = scnarioConf[dnis][s.session[request.Token].Count].Speed
			response.Data.VoiceName = scnarioConf[dnis][s.session[request.Token].Count].VoiceName
			response.Data.Volume = scnarioConf[dnis][s.session[request.Token].Count].Volume
			response.Data.Pitch = scnarioConf[dnis][s.session[request.Token].Count].Pitch
			response.Data.MaxWaitTime = scnarioConf[dnis][s.session[request.Token].Count].MaxWaitTime
			response.Data.BargeIn = scnarioConf[dnis][s.session[request.Token].Count].BargeIn
			response.Data.SttMaxTime = scnarioConf[dnis][s.session[request.Token].Count].SttMaxTime
			response.Data.MaxDigit = scnarioConf[dnis][s.session[request.Token].Count].MaxDigit
			response.Data.EndCharacter = []string{scnarioConf[dnis][s.session[request.Token].Count+1].EndCharacter}
			response.Data.DigitTerm = scnarioConf[dnis][s.session[request.Token].Count].DigitTerm
			response.Data.RecodingFile = "Y"

			if s.session[request.Token].LoopCount > 100 {
				s.session[request.Token] = ResStatus{LoopCount: s.session[request.Token].LoopCount, Count: s.session[request.Token].Count + 1, Status: scnarioConf[dnis][s.session[request.Token].Count+1].Action}
			} else {
				s.session[request.Token] = ResStatus{LoopCount: s.session[request.Token].LoopCount + 1, Count: 1, Status: scnarioConf[dnis][s.session[request.Token].Count].Action}
			}
		} else {
			response.ResultCode = 200
			response.Token = request.Token
			response.Action = scnarioConf[dnis][1].Action
			response.AnounceMents = scnarioConf[dnis][1].AnounceMents
			response.AnnounceFilePath = scnarioConf[dnis][1].AudioFilePath
			response.Data.Speed = scnarioConf[dnis][1].Speed
			response.Data.VoiceName = scnarioConf[dnis][1].VoiceName
			response.Data.Volume = scnarioConf[dnis][1].Volume
			response.Data.Pitch = scnarioConf[dnis][1].Pitch
			response.Data.MaxWaitTime = scnarioConf[dnis][1].MaxWaitTime
			response.Data.BargeIn = scnarioConf[dnis][1].BargeIn
			response.Data.SttMaxTime = scnarioConf[dnis][1].SttMaxTime
			response.Data.MaxDigit = scnarioConf[dnis][1].MaxDigit
			response.Data.EndCharacter = []string{scnarioConf[dnis][1].EndCharacter}
			response.Data.DigitTerm = scnarioConf[dnis][1].DigitTerm
			response.Data.RecodingFile = "Y"

			//	s.m.Lock()
			s.session[request.Token] = ResStatus{Count: s.session[request.Token].Count + 1, Status: scnarioConf[dnis][1].Action}
			//s.m.Unlock()
		}
	} else {
		// fmt.Println("RECV METHOD")
		// fmt.Println(dnis)
		// // if dnis != "9012" && dnis != "9013" && dnis != "9014" {
		// // 	dnis = "sim"
		// // } else
		// if dnis == "AISB" || dnis == "07075999956" || dnis == "821462" || dnis == "1462" {
		// 	dnis = "9013"
		// }
		// fmt.Println(dnis)
		// if dnis == "9012" || dnis == "07012345678" || dnis == "07077442711" {
		// 	dnis = "sim3"
		// }

		// if dnis == "9013" {
		// 	dnis = "9014"
		// }

		// fmt.Println(scnarioConf[dnis])
		switch request.Method {
		case "INIT":
			l.Printf(icslog.LOG_LEVEL_INFO, -1, "INIT..scnarioConf[dnis][1].AnounceMents: %s", scnarioConf[dnis][1].AnounceMents)
			token := createToken(request.CallId)
			response.ResultCode = 200
			response.Token = token
			response.Action = scnarioConf[dnis][1].Action
			response.AnounceMents = scnarioConf[dnis][1].AnounceMents
			response.Data.Speed = scnarioConf[dnis][1].Speed
			response.Data.VoiceName = scnarioConf[dnis][1].VoiceName
			response.Data.Volume = scnarioConf[dnis][1].Volume
			response.Data.Pitch = scnarioConf[dnis][1].Pitch
			response.Data.MaxWaitTime = scnarioConf[dnis][1].MaxWaitTime
			response.Data.BargeIn = scnarioConf[dnis][1].BargeIn
			response.Data.SttMaxTime = scnarioConf[dnis][1].SttMaxTime
			response.Data.MaxDigit = scnarioConf[dnis][1].MaxDigit
			response.Data.EndCharacter = []string{scnarioConf[dnis][1].EndCharacter}
			response.Data.DigitTerm = scnarioConf[dnis][1].DigitTerm
			response.AnnounceFilePath = scnarioConf[dnis][1].AudioFilePath
			response.Data.RecodingFile = "Y"
			//s.m.Lock()
			s.session[token] = ResStatus{Count: 1, Status: scnarioConf[dnis][0].Action, NotUnderstand: 0, PreEventNum: 1}
			// //	s.m.Unlock()
			// case "STT", "DTMF", "BOTH", "MEMO", "NONE":
			// 	l.Printf(icslog.LOG_LEVEL_INFO, -1, "여기는 언제 타는거지지")
			// 	response.ResultCode = 200
			// 	response.Token = request.Token
			// 	if dnis == "9013" || dnis == "9014" {
			// 		response.Action = scnarioConf[dnis][s.session[request.Token].Count+1].Action
			// 		response.AnounceMents = scnarioConf[dnis][s.session[request.Token].Count+1].AnounceMents
			// 		response.Data.Speed = scnarioConf[dnis][s.session[request.Token].Count+1].Speed
			// 		response.Data.VoiceName = scnarioConf[dnis][s.session[request.Token].Count+1].VoiceName
			// 		response.Data.Volume = scnarioConf[dnis][s.session[request.Token].Count+1].Volume
			// 		response.Data.Pitch = scnarioConf[dnis][s.session[request.Token].Count+1].Pitch
			// 		response.Data.MaxWaitTime = scnarioConf[dnis][s.session[request.Token].Count+1].MaxWaitTime
			// 		response.Data.BargeIn = scnarioConf[dnis][s.session[request.Token].Count+1].BargeIn
			// 		response.Data.SttMaxTime = scnarioConf[dnis][s.session[request.Token].Count+1].SttMaxTime
			// 		response.Data.MaxDigit = scnarioConf[dnis][s.session[request.Token].Count+1].MaxDigit
			// 		response.Data.EndCharacter = []string{scnarioConf[dnis][s.session[request.Token].Count+1].EndCharacter}
			// 		response.Data.DigitTerm = scnarioConf[dnis][s.session[request.Token].Count+1].DigitTerm
			// 		response.AnnounceFilePath = scnarioConf[dnis][s.session[request.Token].Count+1].AudioFilePath

			// 		response.Data.RecodingFile = "Y"
			// 		s.session[request.Token] = ResStatus{Count: s.session[request.Token].Count + 1, Status: scnarioConf[dnis][s.session[request.Token].Count+1].Action}
			// 		//	s.m.Unlock()
			// 	} else if dnis == "9012" {
			// 		response.Action = scnarioConf[dnis][1].Action
			// 		response.AnounceMents = scnarioConf[dnis][1].AnounceMents
			// 		response.Data.Speed = scnarioConf[dnis][1].Speed
			// 		response.Data.VoiceName = scnarioConf[dnis][1].VoiceName
			// 		response.Data.Volume = scnarioConf[dnis][1].Volume
			// 		response.Data.Pitch = scnarioConf[dnis][1].Pitch
			// 		response.Data.MaxWaitTime = scnarioConf[dnis][1].MaxWaitTime
			// 		response.Data.BargeIn = scnarioConf[dnis][1].BargeIn
			// 		response.Data.SttMaxTime = scnarioConf[dnis][1].SttMaxTime
			// 		response.Data.MaxDigit = scnarioConf[dnis][1].MaxDigit
			// 		// response.Data.EndCharacter = []string{scnarioConf[dnis][1].EndCharacter}
			// 		// response.Data.DigitTerm = scnarioConf[dnis][s.session[request.Token].Count+1].DigitTerm
			// 		response.Data.RecodingFile = "Y"

			// 		//	s.m.Lock()
			// 		s.session[request.Token] = ResStatus{Count: s.session[request.Token].Count + 1, Status: scnarioConf[dnis][1].Action}
			// 		//	s.m.Unlock()
			// 	} else if dnis == "1000" {
			// 		preEventNum := s.session[request.Token].PreEventNum
			// 		fmt.Printf("\n num %+v\n", preEventNum)
			// 		switch preEventNum {
			// 		case 1:
			// 			actionNum, err := strconv.Atoi(request.TalkText)
			// 			if err == nil {
			// 				response.Action = scnarioConf[dnis][actionNum+2].Action
			// 				response.AnounceMents = scnarioConf[dnis][actionNum+2].AnounceMents
			// 				response.Data.Speed = scnarioConf[dnis][actionNum+2].Speed
			// 				response.Data.VoiceName = scnarioConf[dnis][actionNum+2].VoiceName
			// 				response.Data.Volume = scnarioConf[dnis][actionNum+2].Volume
			// 				response.Data.Pitch = scnarioConf[dnis][actionNum+2].Pitch
			// 				response.Data.MaxWaitTime = scnarioConf[dnis][actionNum+2].MaxWaitTime
			// 				response.Data.BargeIn = scnarioConf[dnis][actionNum+2].BargeIn
			// 				response.Data.SttMaxTime = scnarioConf[dnis][actionNum+2].SttMaxTime
			// 				response.Data.MaxDigit = scnarioConf[dnis][actionNum+2].MaxDigit
			// 				response.Data.EndCharacter = []string{scnarioConf[dnis][actionNum+2].EndCharacter}
			// 				response.Data.DigitTerm = scnarioConf[dnis][actionNum+2].DigitTerm
			// 				response.Data.RecodingFile = "Y"

			// 				//	s.m.Lock()
			// 				s.session[request.Token] = ResStatus{Count: s.session[request.Token].Count + 1, Status: scnarioConf[dnis][actionNum+2].Action, PreEventNum: actionNum + 2}
			// 				//		s.m.Unlock()
			// 			} else {
			// 				response.Action = scnarioConf[dnis][1].Action
			// 				response.AnounceMents = scnarioConf[dnis][1].AnounceMents
			// 				response.Data.Speed = scnarioConf[dnis][1].Speed
			// 				response.Data.VoiceName = scnarioConf[dnis][1].VoiceName
			// 				response.Data.Volume = scnarioConf[dnis][1].Volume
			// 				response.Data.Pitch = scnarioConf[dnis][1].Pitch
			// 				response.Data.MaxWaitTime = scnarioConf[dnis][1].MaxWaitTime
			// 				response.Data.BargeIn = scnarioConf[dnis][1].BargeIn
			// 				response.Data.SttMaxTime = scnarioConf[dnis][1].SttMaxTime
			// 				response.Data.MaxDigit = scnarioConf[dnis][1].MaxDigit
			// 				response.Data.EndCharacter = []string{scnarioConf[dnis][1].EndCharacter}
			// 				response.Data.DigitTerm = scnarioConf[dnis][1].DigitTerm
			// 				response.Data.RecodingFile = "Y"

			// 				//	s.m.Lock()
			// 				s.session[request.Token] = ResStatus{Count: s.session[request.Token].Count + 1, Status: scnarioConf[dnis][1].Action, PreEventNum: 1}
			// 				//	s.m.Unlock()
			// 			}
			// 		case 2:
			// 			actionNum, err := strconv.Atoi(request.TalkText)
			// 			if err == nil {
			// 				if actionNum == 1 {
			// 					response.Action = scnarioConf[dnis][s.session[request.Token].PreEventNum2].Action
			// 					response.AnounceMents = scnarioConf[dnis][s.session[request.Token].PreEventNum2].AnounceMents
			// 					response.Data.Speed = scnarioConf[dnis][s.session[request.Token].PreEventNum2].Speed
			// 					response.Data.VoiceName = scnarioConf[dnis][s.session[request.Token].PreEventNum2].VoiceName
			// 					response.Data.Volume = scnarioConf[dnis][s.session[request.Token].PreEventNum2].Volume
			// 					response.Data.Pitch = scnarioConf[dnis][s.session[request.Token].PreEventNum2].Pitch
			// 					response.Data.MaxWaitTime = scnarioConf[dnis][s.session[request.Token].PreEventNum2].MaxWaitTime
			// 					response.Data.BargeIn = scnarioConf[dnis][s.session[request.Token].PreEventNum2].BargeIn
			// 					response.Data.SttMaxTime = scnarioConf[dnis][s.session[request.Token].PreEventNum2].SttMaxTime
			// 					response.Data.MaxDigit = scnarioConf[dnis][s.session[request.Token].PreEventNum2].MaxDigit
			// 					response.Data.EndCharacter = []string{scnarioConf[dnis][s.session[request.Token].PreEventNum2].EndCharacter}
			// 					response.Data.DigitTerm = scnarioConf[dnis][s.session[request.Token].PreEventNum2].DigitTerm
			// 					response.Data.RecodingFile = "Y"

			// 					//		s.m.Lock()
			// 					s.session[request.Token] = ResStatus{Count: s.session[request.Token].Count + 1, Status: scnarioConf[dnis][s.session[request.Token].PreEventNum2].Action, PreEventNum: s.session[request.Token].PreEventNum2}
			// 					//		s.m.Unlock()
			// 				} else {
			// 					response.Action = scnarioConf[dnis][1].Action
			// 					response.AnounceMents = scnarioConf[dnis][1].AnounceMents
			// 					response.Data.Speed = scnarioConf[dnis][1].Speed
			// 					response.Data.VoiceName = scnarioConf[dnis][1].VoiceName
			// 					response.Data.Volume = scnarioConf[dnis][1].Volume
			// 					response.Data.Pitch = scnarioConf[dnis][1].Pitch
			// 					response.Data.MaxWaitTime = scnarioConf[dnis][1].MaxWaitTime
			// 					response.Data.BargeIn = scnarioConf[dnis][1].BargeIn
			// 					response.Data.SttMaxTime = scnarioConf[dnis][1].SttMaxTime
			// 					response.Data.MaxDigit = scnarioConf[dnis][1].MaxDigit
			// 					response.Data.EndCharacter = []string{scnarioConf[dnis][1].EndCharacter}
			// 					response.Data.DigitTerm = scnarioConf[dnis][1].DigitTerm
			// 					response.Data.RecodingFile = "Y"

			// 					//		s.m.Lock()
			// 					s.session[request.Token] = ResStatus{Count: s.session[request.Token].Count + 1, Status: scnarioConf[dnis][1].Action, PreEventNum: 1, PreEventNum2: 1}
			// 					//	s.m.Unlock()
			// 				}

			// 			} else {
			// 				response.Action = scnarioConf[dnis][1].Action
			// 				response.AnounceMents = scnarioConf[dnis][1].AnounceMents
			// 				response.Data.Speed = scnarioConf[dnis][1].Speed
			// 				response.Data.VoiceName = scnarioConf[dnis][1].VoiceName
			// 				response.Data.Volume = scnarioConf[dnis][1].Volume
			// 				response.Data.Pitch = scnarioConf[dnis][1].Pitch
			// 				response.Data.MaxWaitTime = scnarioConf[dnis][1].MaxWaitTime
			// 				response.Data.BargeIn = scnarioConf[dnis][1].BargeIn
			// 				response.Data.SttMaxTime = scnarioConf[dnis][1].SttMaxTime
			// 				response.Data.MaxDigit = scnarioConf[dnis][1].MaxDigit
			// 				response.Data.EndCharacter = []string{scnarioConf[dnis][1].EndCharacter}
			// 				response.Data.DigitTerm = scnarioConf[dnis][1].DigitTerm
			// 				response.Data.RecodingFile = "Y"

			// 				//			s.m.Lock()
			// 				s.session[request.Token] = ResStatus{Count: s.session[request.Token].Count + 1, Status: scnarioConf[dnis][1].Action, PreEventNum: 1}
			// 				//		s.m.Unlock()
			// 			}
			// 		default:

			// 			response.Action = scnarioConf[dnis][2].Action
			// 			response.AnounceMents = "결과 " + request.TalkText + " " + scnarioConf[dnis][2].AnounceMents
			// 			response.Data.Speed = scnarioConf[dnis][2].Speed
			// 			response.Data.VoiceName = scnarioConf[dnis][2].VoiceName
			// 			response.Data.Volume = scnarioConf[dnis][2].Volume
			// 			response.Data.Pitch = scnarioConf[dnis][2].Pitch
			// 			response.Data.MaxWaitTime = scnarioConf[dnis][2].MaxWaitTime
			// 			response.Data.BargeIn = scnarioConf[dnis][2].BargeIn
			// 			response.Data.SttMaxTime = scnarioConf[dnis][2].SttMaxTime
			// 			response.Data.MaxDigit = scnarioConf[dnis][2].MaxDigit
			// 			response.Data.EndCharacter = []string{scnarioConf[dnis][2].EndCharacter}
			// 			response.Data.DigitTerm = scnarioConf[dnis][2].DigitTerm
			// 			response.Data.RecodingFile = "Y"

			// 			//	s.m.Lock()
			// 			s.session[request.Token] = ResStatus{Count: s.session[request.Token].Count + 1, Status: scnarioConf[dnis][2].Action, PreEventNum: 2, PreEventNum2: s.session[request.Token].PreEventNum}
			// 			//s.m.Unlock()
			// 		}
			// 	} else {
			// 		//시뮬레이터
			// 		response.Action = scnarioConf[dnis][s.session[request.Token].Count].Action
			// 		response.AnounceMents = scnarioConf[dnis][s.session[request.Token].Count].AnounceMents
			// 		response.Data.Speed = scnarioConf[dnis][s.session[request.Token].Count].Speed
			// 		response.Data.VoiceName = scnarioConf[dnis][s.session[request.Token].Count].VoiceName
			// 		response.Data.Volume = scnarioConf[dnis][s.session[request.Token].Count].Volume
			// 		response.Data.Pitch = scnarioConf[dnis][s.session[request.Token].Count].Pitch
			// 		response.Data.MaxWaitTime = scnarioConf[dnis][s.session[request.Token].Count].MaxWaitTime
			// 		response.Data.BargeIn = scnarioConf[dnis][s.session[request.Token].Count].BargeIn
			// 		response.Data.SttMaxTime = scnarioConf[dnis][s.session[request.Token].Count].SttMaxTime
			// 		response.Data.MaxDigit = scnarioConf[dnis][s.session[request.Token].Count].MaxDigit
			// 		response.Data.EndCharacter = []string{scnarioConf[dnis][s.session[request.Token].Count+1].EndCharacter}
			// 		response.Data.DigitTerm = scnarioConf[dnis][s.session[request.Token].Count].DigitTerm
			// 		response.Data.RecodingFile = "Y"

			// 		if s.session[request.Token].LoopCount > 100 {
			// 			s.session[request.Token] = ResStatus{LoopCount: s.session[request.Token].LoopCount, Count: s.session[request.Token].Count + 1, Status: scnarioConf[dnis][s.session[request.Token].Count].Action}
			// 		} else {
			// 			s.session[request.Token] = ResStatus{LoopCount: s.session[request.Token].LoopCount + 1, Count: s.session[request.Token].Count, Status: scnarioConf[dnis][s.session[request.Token].Count].Action}
			// 		}
			// 		//			s.m.Lock()
			// 		//		s.m.Unlock()
			// 	}

			// case "REFER":
			// 	response.ResultCode = 200
			// 	response.Token = request.Token
			// 	response.Action = "TRANSFER"
			// 	response.AnounceMents = "네. 매장으로 연결해드릴게요."
			// 	response.Data.BargeIn = ""
			// 	response.Data.RecodingFile = ""
			// 	response.Data.SttMaxTime = 0
			// 	response.Data.TelNo = request.TelNo

			// 	//	s.m.Lock()
			// 	s.session[request.Token] = ResStatus{Count: s.session[request.Token].Count + 1, Status: scnarioConf[dnis][s.session[request.Token].Count+1].Action}
			// case "TRANSFER":
			// 	response.ResultCode = 200
			// 	response.Token = request.Token
			// 	response.Action = "TRANSFER"
			// 	response.AnounceMents = "네. 매장으로 연결해드릴게요."
			// 	response.Data.BargeIn = ""
			// 	response.Data.RecodingFile = ""
			// 	response.Data.SttMaxTime = 0
			// 	response.Data.TelNo = request.TelNo

			// 	//	s.m.Lock()
			// 	s.session[request.Token] = ResStatus{Count: s.session[request.Token].Count + 1, Status: scnarioConf[dnis][s.session[request.Token].Count+1].Action}
			// //	s.m.Unlock()

			// case "TRNASFER2": // 전화 종료 요청
			// 	response.ResultCode = 200
			// 	response.Token = request.Token
			// 	response.Action = "TRANSFER2"
			// 	response.AnounceMents = "네. 매장으로 연결해드릴게요."
			// 	response.Data.BargeIn = ""
			// 	response.Data.RecodingFile = ""
			// 	response.Data.SttMaxTime = 0
			// 	response.Data.TelNo = request.TelNo

			// 	//      s.m.Lock()
			// 	s.session[request.Token] = ResStatus{Count: s.session[request.Token].Count + 1, Status: scnarioConf[dnis][s.session[request.Token].Count+1].Action}
			// 	//      s.m.Unlock()

			// 	response.Token = request.Token
			// 	response.Action = "END"
			// 	response.AnounceMents = "접수가 완료되었습니다."
			// 	response.Data.BargeIn = "N"
			// 	response.Data.RecodingFile = "Y"
			// 	response.Data.SttMaxTime = 0
			// }

		}

		response.Data.UUI = json.RawMessage{}
		uuiMap := map[string]interface{}{
			"test1": "testdata1231",
			"test2": 123,
			"test3": 0.1,
		}

		// 맵을 JSON으로 마샬링
		uuiJSON, err := json.Marshal(uuiMap)
		if err != nil {
			fmt.Println("Error marshaling UUI:", err)
			return
		}

		// 마샬링된 JSON을 json.RawMessage로 변환
		response.Data.UUI = json.RawMessage(uuiJSON)
		response.Data.TelNo = request.TelNo
		s.m.Unlock()
		// response marshal

		resMarshal, jerr := json.Marshal(response)
		if jerr != nil {
			fmt.Println("Json Marshal error ", jerr)
		}
		fmt.Printf("%+v", response)
		l.Printf(icslog.LOG_LEVEL_INFO, -1, "Send Message\n %s", string(resMarshal))

		// if s.session[request.Token].Count < len(scnarioConf[dnis]) && scnarioConf[dnis][s.session[request.Token].Count].TimeValue {
		// 	l.Printf(icslog.LOG_LEVEL_INFO, -1, "Time Sleep %d \r\n", scnarioConf[dnis][s.session[request.Token].Count].TimeSleep)
		// 	time.Sleep(time.Second * time.Duration(time.Duration(scnarioConf[dnis][s.session[request.Token].Count].TimeSleep)))
		// } else {

		// }
		time.Sleep(time.Second * 1)
		// send response
		fmt.Fprintln(w, string(resMarshal))

	}
}

func createToken(callId string) string {
	token := ""
	times := fmt.Sprintf("%d", time.Now().UnixNano()/1000000)
	tokens := strings.Split(callId, "@")[0] + times
	token = tokens[0:8] + "-" + tokens[8:10] + "-" + tokens[10:23]
	return token
}

func cryptoRandSecure(max int64) int64 {
	nBig, err := rand.Int(rand.Reader, big.NewInt(max))
	if err != nil {
		fmt.Println(err)
	}
	return nBig.Int64()
}

func (s *ScenarioSession) ByeCheck(method string, token string) {
	l := icslog.GetIcsLog()
	// fmt.Printf(">>>>>> SESSION BYE INFO - token [%s]\n", token)
	l.Printf(icslog.LOG_LEVEL_INFO, -1, ">>>>>> SESSION BYE INFO - token [%s] ", token)
	s.m.Lock()
	delete(s.session, token)
	s.m.Unlock()
}
